#######################################################################################
#
#                             - Measuring Mobility -
#    
# by Frank A. Cowell and E. Flachaire, October 2017
#
# replication file: Table 2, 3, 4 and 5, Monte Carlo simulations
#
#######################################################################################
rm(list=ls(all=TRUE))

######################### User selection 

nobs=100     # select the sample size
rho=0        # select the correlation level between the two samples (0<=rho<1)
status=0     # select the type of mobility: 0=income mobility, 1=rank mobility

nboot=199    # number of bootstrap samples
nrepet=1000  # number of replications

#########################

source("./mobility.r")

library(MASS)

if (status==0) alpha=c(-1,0,.5,1,2)
if (status==1) alpha=c(-0.5,0,.5,1,1.5)


# Define the income generating process and, approximate the true value of the index from a huge sample
Sigma<-matrix(c(1,rho,rho,1),2,2)
Mu <- c(0,0)
inc0 <- exp(mvrnorm(n=1000000, Mu, Sigma))
b0 = M.stat(inc0[,1],inc0[,2],alpha,status=status,se=FALSE)$index

# set some values
crit=qt(.975,nobs-1)
nalpha=NROW(alpha)

Mbootp = matrix(0,nboot,nalpha)       # bootstrap percentile
Mboots = matrix(0,nboot,nalpha)       # bootstrap studentized

failRasymp=matrix(0,nalpha)
failLasymp=matrix(0,nalpha)
failRboot=matrix(0,nalpha)
failLboot=matrix(0,nalpha)
failRbootp=matrix(0,nalpha)
failLbootp=matrix(0,nalpha)

for (i in 1:nrepet) {

  cat("simu loop: ", i, "\r")
  
  inc <- exp(mvrnorm(n=nobs, Mu, Sigma))                   # generate two income samples
  Msimu <- M.stat(inc[,1],inc[,2],a=alpha,status=status)   # compute mobility indices

  if (status==0)  {             # asymptotic confidence intervals for income mobility (status=0)
    for (j in 1:nalpha) {   
      if ( b0[j] < Msimu$index[j] - crit*Msimu$se[j] )  failLasymp[j] = failLasymp[j] + 1/nrepet
      if ( b0[j] > Msimu$index[j] + crit*Msimu$se[j] )  failRasymp[j] = failRasymp[j] + 1/nrepet
    }
  }
  
  if (nboot>0) {                # bootstrap confidence intervals
    for (b in 1:nboot) {      
      iboot=sample(nobs,replace=T)     
      Mb <- M.stat(inc[iboot,1],inc[iboot,2],a=alpha,status=status)
      Mbootp[b,]=Mb$index
      if (status==0) Mboots[b,]=(Mb$index-Msimu$index)/Mb$se    
    }
    for (j in 1:nalpha) {     # bootstrap percentile CI for income & rank mobility (status=0,1)
      critb1p = quantile(Mbootp[,j],probs=c(.975),type=1,names=F)
      critb2p = quantile(Mbootp[,j],probs=c(.025),type=1,names=F)
      if ( b0[j] < critb2p )  failLbootp[j] = failLbootp[j] + 1/nrepet
      if ( b0[j] > critb1p )  failRbootp[j] = failRbootp[j] + 1/nrepet   
      if (status==0) {       # bootstrap studentized CI for income mobility (status=0)
        critb1 = quantile(Mboots[,j],probs=c(.025),type=1,names=F)
        critb2 = quantile(Mboots[,j],probs=c(.975),type=1,names=F)
        if ( (Msimu$index[j]-b0[j])/Msimu$se[j] < critb1 )  failLboot[j] = failLboot[j] + 1/nrepet
        if ( (Msimu$index[j]-b0[j])/Msimu$se[j] > critb2 )  failRboot[j] = failRboot[j] + 1/nrepet
      }
    }  
  }
     
}

######## Print the results

asymp=as.numeric(failLasymp+failRasymp)
bootp=as.numeric(failLbootp+failRbootp)
boot=as.numeric(failLboot+failRboot)
if (status==0) {res=t(cbind(asymp,bootp,boot)) ; rownames(res)=c("Asymptotic", "Boot-perc", "Boot-stud")}
if (status==1) {res=t(cbind(bootp)) ; rownames(res)=c("Boot-perc")}
colnames(res)=alpha


if (status==0) type_mobility="Income mobility" else if (status==1) type_mobility="Rank mobility"
cat('\n', type_mobility, ': rho=',rho,' nobs=',nobs,' nrepet=',nrepet, ' nboot=',nboot, '\n')
res




