#######################################################################################
#
#                             - Measuring Mobility -
#    
# by Frank A. Cowell and E. Flachaire, October 2017
#
# replication file: Table 6, Comparisons of mobility indices in different scenarios
#
#######################################################################################
rm(list=ls())

################### Define mobility indices

# Elasticity and correlation coefficients
H=function(u,v) 1-cov(log(u),log(v))/var(log(u))
Icorr=function(u,v) 1-cor(log(u),log(v))
Rcorr=function(u,v) 1-cor(u,v,method="spearman")

# Fields and Ok (1996,1999)
FO1=function(u,v) sum(abs(v-u))/NROW(u)
FO2=function(u,v) sum(abs(log(v)-log(u)))/NROW(u)

# Shorrocks (78)
S1=function(u,v) {
  library(ineq)
  w1=mean(u)/mean(u+v)
  w2=mean(v)/mean(u+v)
  1-Theil(u+v)/(w1*Theil(u)+w2*Theil(v))
}
S2=function(u,v) {
  library(ineq)
  w1=mean(u)/mean(u+v)
  w2=mean(v)/mean(u+v)
  1-Gini(u+v)/(w1*Gini(u)+w2*Gini(v))
}

# Cowell and Flachaire (2017)
CF=function(u,v,a=0, type="income", axiom="scale") {
  # u and v : income in period 0 and in period 1
  # a: sensitivity parameter
  # type: income or rank mobility index
  # axiom: scale- or translation-irrelevant axiom
  if (type=="rank") {
    u=rank(u)/NROW(u); v=rank(v)/NROW(v)}
  mu=mean(u); mv=mean(v); n=NROW(u)
  if (axiom=="scale") {
    if (a==0) m=-sum((v/mv)*log((u/mu)/(v/mv)))/n
    else if (a==1) m=sum((u/mu)*log((u/mu)/(v/mv)))/n
    else m=sum((u/mu)^a*(v/mv)^(1-a)-1)/(a*(a-1)*n)  }
  if (axiom=="translation") {
    if (a==0) m=var(u-v)/2
    else m=sum(exp(a*(u-mu-v+mv))-1)/((n-1)*a^2)}
  m
}


################### Different scenarios

x1=exp(c(1,1.5,2))
x2=x1*2
x3=x1+2
x4=exp(c(1.5,2,1))
x5=x4*2
x6=x4+2
x7=exp(c(1,3,2))
x8=exp(c(1,2,3))

################### Compute mobility indices

r1=c(H(x1,x2),H(x1,x3),H(x1,x4),H(x1,x5),H(x1,x6),H(x1,x7),H(x1,x8))
r2=c(Icorr(x1,x2),Icorr(x1,x3),Icorr(x1,x4),Icorr(x1,x5),Icorr(x1,x6),Icorr(x1,x7),Icorr(x1,x8))
r3=c(CF(x1,x2),CF(x1,x3),CF(x1,x4),CF(x1,x5),CF(x1,x6),CF(x1,x7),CF(x1,x8))
r4=c(CF(x1,x2,axiom="translation"),CF(x1,x3,axiom="translation"),CF(x1,x4,axiom="translation"),CF(x1,x5,axiom="translation"),CF(x1,x6,axiom="translation"),CF(x1,x7,axiom="translation"),CF(x1,x8,axiom="translation"))
r5=c(FO1(x1,x2),FO1(x1,x3),FO1(x1,x4),FO1(x1,x5),FO1(x1,x6),FO1(x1,x7),FO1(x1,x8))
r6=c(FO2(x1,x2),FO2(x1,x3),FO2(x1,x4),FO2(x1,x5),FO2(x1,x6),FO2(x1,x7),FO2(x1,x8))
r7=c(S1(x1,x2),S1(x1,x3),S1(x1,x4),S1(x1,x5),S1(x1,x6),S1(x1,x7),S1(x1,x8))
r8=c(S2(x1,x2),S2(x1,x3),S2(x1,x4),S2(x1,x5),S2(x1,x6),S2(x1,x7),S2(x1,x8))
r9=c(Rcorr(x1,x2),Rcorr(x1,x3),Rcorr(x1,x4),Rcorr(x1,x5),Rcorr(x1,x6),Rcorr(x1,x7),Rcorr(x1,x8))
r10=c(CF(x1,x2,type="rank"),CF(x1,x3,type="rank"),CF(x1,x4,type="rank"),CF(x1,x5,type="rank"),CF(x1,x6,type="rank"),CF(x1,x7,type="rank"),CF(x1,x8,type="rank"))

################### Print the results

res=cbind(r1,r2,r3,r4,r5,r6,r7,r8)
colnames(res)=c("Elasticity", "Pearson corr.", "Cowell-Flachaire 1", "Cowell-Flachaire 2", "Fields-Ok 1", "Fields-Ok 1", "Shorrocks 1", "Shorrocks 2")
rownames(res)=c("1a","1b","1c","1d","1e","1f","1g")
round(t(res),3)

res=cbind(r9,r10)
colnames(res)=c("Spearman corr.", "Cowell-Flachaire")
rownames(res)=c("1a","1b","1c","1d","1e","1f","1g")
round(t(res),3)

